package com.yupi.springbootinit.manager;

import org.apache.poi.ss.formula.functions.T;
import org.jetbrains.annotations.NotNull;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

import java.util.concurrent.ArrayBlockingQueue;
import java.util.concurrent.ThreadFactory;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

@Configuration
public class ThreadPoolExecutorConfig {

    @Bean
    public ThreadPoolExecutor threadPoolExecutor() {
        ThreadFactory threadFactory = new ThreadFactory() {
            private int count = 1;

            @Override
            public Thread newThread(@NotNull Runnable r) {
                Thread thread = new Thread(r);
                thread.setName("线程" + count);
                count++;
                return thread;
            }
        };

//        corePoolSize(核心线程数=>正式员工数)：正常情况下，我们的系统应该同时工作的线程数（随时就绪的状态）
//        maximumPoolSize(最大线程数=>哪怕任务再多，你也最多招这些人)：极限情况下，我们的线程池最多有多少个线程？
//        keepAliveTime(空闲线程存活时间)：非核心线程在没有任务的情况下，过多久要删除（理解为开除临时工），从而释放无用的线程资源。
//        TimeUnit unit(空闲线程存活时间的单位)：分钟、秒
//        workQueue(工作队列)：用于存放给线程执行的任务，存在一个队列的长度（一定要设置，不要说队列长度无限，因为也会占用资源）
//        threadFactory(线程工厂)：控制每个线程的生成、线程的属性（比如线程名）
        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(
                2,
                4,
                100,
                TimeUnit.SECONDS,
                new ArrayBlockingQueue<>(4),
                threadFactory);
        return threadPoolExecutor;
    }
}